##
#     Project: PyZendesk
# Description: API for Zendesk
#      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
#   Copyright: 2021-2025 Fabio Castelli
#     License: GPL-3+
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
##

import logging
from typing import Optional

import requests
import requests.auth


class Api(object):
    def __init__(self, website: str):
        self.website = website[:-1] if website.endswith('/') else website
        self.username = None
        self.password = None

    def authenticate(self, username: str, password: str) -> None:
        """
        Set authentication username and password

        :param username: user name for login
        :param password: user password for login
        :return: None
        """
        self.username = username
        self.password = password

    def request_raw(self,
                    method: str,
                    path: str,
                    headers: dict,
                    params: Optional[dict],
                    data: Optional[bytes],
                    json: Optional[dict]) -> requests.Response:
        """
        Send a raw REST request to Zendesk

        :param method: REST method to use (get, post, put, delete)
        :param path: API path which will be added to the base API path
        :param headers: dictionary with HTTP headers
        :param params: additional query string to send along with the request
        :param data: additional raw data to send along with the request
        :param json: additional JSON data to send along with the request
        :return: raw requests response
        """
        logging_path = path.replace('\n', '\\n')
        logging.debug(f'Executing {method} request '
                      f'for url {self.website}/api/v2/{logging_path}')
        req = requests.request(method=method,
                               url=f'{self.website}/api/v2/{path}',
                               auth=requests.auth.HTTPBasicAuth(
                                   username=self.username,
                                   password=self.password),
                               headers=headers,
                               params=params,
                               data=data,
                               json=json)
        return req

    def request(self,
                method: str,
                path: str,
                json: Optional[dict]) -> dict:
        """
        Send a JSON REST request to Zendesk

        :param method: REST method to use (get, post, put, delete)
        :param path: API path which will be added to the base API path
        :param json: additional JSON data to send along with the request
        :return: response from JSON data
        """
        req = self.request_raw(method=method,
                               path=path,
                               headers={'Content-Type': 'application/json'},
                               params=None,
                               data=None,
                               json=json)
        return req.json()

    def request_delete(self,
                       path: str) -> dict:
        """
        Send a DELETE REST request to Zendesk

        :param path: API path which will be added to the base API path
        :return: response from JSON data
        """
        return self.request(method='delete', path=path, json=None)

    def request_get(self,
                    path: str) -> dict:
        """
        Send a GET REST request to Zendesk

        :param path: API path which will be added to the base API path
        :return: response from JSON data
        """
        return self.request(method='get', path=path, json=None)

    def request_post(self,
                     path: str,
                     json: dict) -> dict:
        """
        Send a POST REST request to Zendesk

        :param path: API path which will be added to the base API path
        :param json: additional JSON data to send along with the request
        :return: response from JSON data
        """
        return self.request(method='post', path=path, json=json)

    def request_put(self,
                    path: str,
                    json: dict) -> dict:
        """
        Send a PUT REST request to Zendesk

        :param path: API path which will be added to the base API path
        :param json: additional JSON data to send along with the request
        :return: response from JSON data
        """
        return self.request(method='put', path=path, json=json)



##
#     Project: PyZendesk
# Description: API for Zendesk
#      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
#   Copyright: 2021-2025 Fabio Castelli
#     License: GPL-3+
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
##

from typing import Optional

from .api import Api
from .users import Users


class Admins(Api):
    def __init__(self, website: str):
        super().__init__(website=website)
        self._users = Users(website=website)

    def authenticate(self, username: str, password: str) -> None:
        """
        Set authentication username and password

        :param username: user name for login
        :param password: user password for login
        :return: None
        """
        super().authenticate(username=username,
                             password=password)
        self._users.authenticate(username=username,
                                 password=password)

    def me(self) -> dict:
        """
        Zendesk requester admin information

        :return: admin information
        """
        return self._users.me()

    def get(self, admin_id: int) -> dict:
        """
        Get a admin's details

        :param admin_id: admin ID to get data from
        :return: dictionary with the admin details
        """
        return self._users.get(user_id=admin_id)

    def get_many(self, admin_ids: list[int]) -> dict:
        """
        Get many admins' details

        :param admin_ids: list of admins ID to get data from
        :return: dictionary with the admin details
        """
        return self._users.get_many(user_ids=admin_ids)

    def get_related(self, admin_id: int) -> dict:
        """
        Get a admin's related details

        :param admin_id: admin ID to get data from
        :return: dictionary with the admin related details
        """
        return self._users.get_related(user_id=admin_id)

    def count(self) -> Optional[int]:
        """
        Get the number of admins

        :return: number of admins found
        """
        results = self._users.count(criteria_list=['role:admin'])
        return results

    def search(self, criteria_list: list) -> dict:
        """
        Get the admins matching the specified criterias

        :param criteria_list: list of string criterias
        :return: dictionary with admins details found
        """
        criteria_list_copy = criteria_list.copy()
        criteria_list_copy.append('role:admin')
        return self._users.search(criteria_list=criteria_list_copy)

    def search_all(self, criteria_list: list) -> dict:
        """
        Get the admins matching the specified criterias processing all the
        results by requesting also the next pages.

        :param criteria_list: list of string criterias
        :return: dictionary with admins details found
        """
        criteria_list_copy = criteria_list.copy()
        criteria_list_copy.append('role:admin')
        results = self._users.search_all(criteria_list=criteria_list_copy)
        return results



##
#     Project: PyZendesk
# Description: API for Zendesk
#      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
#   Copyright: 2021-2025 Fabio Castelli
#     License: GPL-3+
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
##

import os

from pyzendesk import Attachments as ZendeskAttachments
from pyzendesk import Tickets as ZendeskTickets
from pyzendesk import (TICKET_STATUS_NEW,
                       TICKET_STATUS_OPEN,
                       TICKET_STATUS_PENDING,
                       TICKET_STATUS_HOLD,
                       TICKET_STATUS_SOLVED,
                       TICKET_STATUS_CLOSED)


# Instance zendesk object
zendesk = ZendeskTickets(website=os.environ['ZENDESK_SERVER'])
# Authenticate user
zendesk.authenticate(username=os.environ['ZENDESK_USERNAME'],
                     password=os.environ['ZENDESK_PASSWORD'])

# Get the tickets count from 2021-01-01 to 2021-01-31
count = zendesk.count(criteria_list=['created>=2021-01-01',
                                     'created<=2021-01-31'])
print('tickets count found:', count)

# Get the tickets details from 2021-01-01 to 2021-01-31
tickets = zendesk.search(criteria_list=['created>=2021-01-01',
                                        'created<=2021-01-31'])
print('tickets details:', len(tickets['results']))

# Get all the tickets details from 2021-01-01 to 2021-01-31
tickets = zendesk.search_all(criteria_list=['created>=2021-01-01',
                                            'created<=2021-01-31'])
print('tickets details:', len(tickets['results']))

# Get the tickets details from 2021-01-01 to 2021-01-31 using export API
tickets = zendesk.search_export(criteria_list=['created>=2021-01-01',
                                               'created<=2021-01-31'])
print('tickets details:', len(tickets['results']))

# Get the tickets details from 2021-01-01 to 2021-01-31 using export API
tickets = zendesk.search_export_all(criteria_list=['created>=2021-01-01',
                                                   'created<=2021-01-31'])
print('tickets details:', len(tickets['results']))

# Get details for the first ticket using its ID
ticket_id = tickets['results'][0]['id']
ticket = zendesk.get(ticket_id=ticket_id)
print('ticket details:', ticket)
comments = zendesk.get_comments(ticket_id=ticket_id)
print('ticket comments:', comments)
comments = zendesk.get_comments_all(ticket_id=ticket_id)
print('ticket comments:', comments)

ticket_id = 358004
# Set ticket custom fields
ticket = zendesk.update_custom_fields(ticket_id=ticket_id,
                                      fields={1900004825713: 'something',
                                              1900005530233: 'fr'})
print('ticket details:', ticket)

# Add private comment to a ticket
ticket = zendesk.add_private_comment(ticket_id=ticket_id,
                                     text='This is **private** comment',
                                     attachments=None)
print('ticket details:', ticket)

# Add public comment to a ticket
ticket = zendesk.add_public_comment(ticket_id=ticket_id,
                                    text='This is **public** comment',
                                    attachments=None)
print('ticket details:', ticket)


# Instance attachments object
attachments = ZendeskAttachments(website=os.environ['ZENDESK_SERVER'])
attachments.authenticate(username=os.environ['ZENDESK_USERNAME'],
                         password=os.environ['ZENDESK_PASSWORD'])
with open(__file__, 'rb') as file:
    attachment_content = file.read()

# Add private comment to a ticket with attachments
attachment = attachments.upload(content_type='text/plain',
                                filename='test.py',
                                data=attachment_content)
attachment_token = attachment['upload']['token']
ticket = zendesk.add_private_comment(ticket_id=ticket_id,
                                     text='With attachment',
                                     attachments=[attachment_token],
                                     status=TICKET_STATUS_OPEN)
print('tickets details:', ticket)

# Add public comment to a ticket with attachments
attachment = attachments.upload(content_type='text/plain',
                                filename='test.py',
                                data=attachment_content)
attachment_token = attachment['upload']['token']
ticket = zendesk.add_public_comment(ticket_id=ticket_id,
                                    text='With attachment',
                                    attachments=[attachment_token],
                                    status=TICKET_STATUS_PENDING)
print('ticket details:', ticket)

# Change ticket status
for status in (TICKET_STATUS_NEW,
               TICKET_STATUS_OPEN,
               TICKET_STATUS_PENDING,
               TICKET_STATUS_HOLD,
               TICKET_STATUS_SOLVED,
               TICKET_STATUS_CLOSED):
    ticket = zendesk.set_status(ticket_id=ticket_id,
                                status=status)
    print('ticket details:', ticket)



##
#     Project: PyZendesk
# Description: API for Zendesk
#      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
#   Copyright: 2021-2025 Fabio Castelli
#     License: GPL-3+
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
##

APP_NAME = 'PyZendesk'
APP_VERSION = '0.5.0'
APP_DESCRIPTION = 'API for Zendesk'
APP_DOMAIN = 'pyzendesk'
APP_ID = f'{APP_DOMAIN}.muflone.com'
APP_AUTHOR = 'Fabio Castelli'
APP_AUTHOR_EMAIL = 'muflone@muflone.com'
APP_COPYRIGHT = f'Copyright 2021-2025 {APP_AUTHOR}'
URL_AUTHOR = 'http://www.muflone.com/'
URL_APPLICATION = f'{URL_AUTHOR}{APP_DOMAIN}/'
URL_SOURCES = f'https://github.com/muflone/{APP_DOMAIN}/'



##
#     Project: PyZendesk
# Description: API for Zendesk
#      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
#   Copyright: 2021-2025 Fabio Castelli
#     License: GPL-3+
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
##

from .api import Api


class Attachments(Api):
    def upload(self, content_type: str, filename: str, data: bytes) -> dict:
        """
        Upload an attachment using the specified content type

        :param content_type: HTTP content_type
        :param filename: filename for the uploaded file
        :param data: raw data to upload
        :return: upload JSON results
        """
        return self.request_raw(method='post',
                                path='uploads.json',
                                headers={'Content-Type': content_type},
                                params={'filename': filename},
                                data=data,
                                json=None).json()



##
#     Project: PyZendesk
# Description: API for Zendesk
#      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
#   Copyright: 2021-2025 Fabio Castelli
#     License: GPL-3+
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
##

import json
import os

from pyzendesk import Admins as ZendeskAdmins


# Instance zendesk object
zendesk = ZendeskAdmins(website=os.environ['ZENDESK_SERVER'])
# Authenticate user
zendesk.authenticate(username=os.environ['ZENDESK_USERNAME'],
                     password=os.environ['ZENDESK_PASSWORD'])

# Get the admins count
count = zendesk.count()
print('admins count found:', count)

# Search
admins = zendesk.search(criteria_list=[])
print('admins details:', len(admins['users']))
# Search all
admins = zendesk.search_all(criteria_list=[])
print('admins details:', len(admins['users']))

# Get admin details
admin = zendesk.get(admin_id=admins['users'][0]['id'])
print(json.dumps(obj=admin,
                 indent=4))

# Get admin related details
admin = zendesk.get_related(admin_id=admins['users'][0]['id'])
print(json.dumps(obj=admin,
                 indent=4))

# Get many admins
admins = zendesk.get_many(admin_ids=[admins['users'][0]['id'],
                                     admins['users'][1]['id'],
                                     admins['users'][2]['id']])
print('admins details:', len(admins['users']))



##
#     Project: PyZendesk
# Description: API for Zendesk
#      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
#   Copyright: 2021-2025 Fabio Castelli
#     License: GPL-3+
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
##

import json
import os

from pyzendesk import constants
from pyzendesk import Users as ZendeskUsers


# Instance zendesk object
zendesk = ZendeskUsers(website=os.environ['ZENDESK_SERVER'])
# Authenticate user
zendesk.authenticate(username=os.environ['ZENDESK_USERNAME'],
                     password=os.environ['ZENDESK_PASSWORD'])

# Get the users count from 2021-01-01 to 2021-01-31
count = zendesk.count(criteria_list=['created>=2021-01-01',
                                     'created<=2021-01-31'])
print('users count found:', count)

# Get the users details from 2021-01-01 to 2021-01-31
users = zendesk.search(criteria_list=['created>=2021-01-01',
                                      'created<=2021-01-31'])
print('users details:', len(users['users']))

# Get all the users details from 2021-01-01 to 2021-01-31
users = zendesk.search_all(criteria_list=['created>=2021-01-01',
                                          'created<=2021-01-31'])
print('users details:', len(users['users']))

# Get user details
user = zendesk.get(user_id=users['users'][0]['id'])
print(json.dumps(obj=user,
                 indent=4))

# Get user related details
user = zendesk.get_related(user_id=users['users'][0]['id'])
print(json.dumps(obj=user,
                 indent=4))

# Autocomplete users
users = zendesk.autocomplete(name='Fabio C')
print('users details:', len(users['users']))

# Autocomplete all users
users = zendesk.autocomplete_all(name='Fabio C')
print('users details:', len(users['users']))

# Get many users
users = zendesk.get_many(user_ids=[users['users'][0]['id'],
                                   users['users'][1]['id'],
                                   users['users'][2]['id']])
print('users details:', len(users['users']))

# Create new user
user = zendesk.create(user={'email': constants.APP_AUTHOR_EMAIL,
                            'name': constants.APP_AUTHOR,
                            'role': 'end-user',
                            'verified': True})
print(json.dumps(obj=user,
                 indent=4))

# Merge two users
user2 = zendesk.create(user={'email': f'{constants.APP_AUTHOR_EMAIL}.copy',
                             'name': constants.APP_AUTHOR,
                             'role': 'end-user',
                             'verified': True})
user = zendesk.merge(user_id=user2['user']['id'],
                     user_id_final=user['user']['id'])

# Create or update user
user = zendesk.create_or_update(user={'email': constants.APP_AUTHOR_EMAIL,
                                      'name': constants.APP_AUTHOR,
                                      'role': 'end-user',
                                      'verified': True,
                                      'phone': '+39987654321'})

# Update user
user = zendesk.update(user_id=user['user']['id'],
                      user={'phone': '+39123456789'})

# Delete the new user
user = zendesk.delete(user_id=user['user']['id'])
print(json.dumps(obj=user,
                 indent=4))

# List deleted users
users = zendesk.list_deleted()
print('users details:', len(users['deleted_users']))

# Get the newly deleted user
user = zendesk.get_deleted(user_id=user['user']['id'])
print(json.dumps(obj=user,
                 indent=4))

# Purge the new user
user = zendesk.purge(user_id=user['deleted_user']['id'])
print(json.dumps(obj=user,
                 indent=4))

# Create two new users
# status = zendesk.create_many(users=[
#     {'email': constants.APP_AUTHOR_EMAIL,
#      'name': constants.APP_AUTHOR,
#      'role': 'end-user',
#      'verified': True},
#     {'email': f'{constants.APP_AUTHOR_EMAIL}.copy',
#      'name': constants.APP_AUTHOR,
#      'role': 'end-user',
#      'verified': True},
#     ])
# print(json.dumps(obj=status,
#                  indent=4))

# Create two new users
# status = zendesk.create_or_update_many(users=[
#     {'email': constants.APP_AUTHOR_EMAIL,
#      'name': constants.APP_AUTHOR,
#      'role': 'end-user',
#      'verified': True},
#     {'email': f'{constants.APP_AUTHOR_EMAIL}.copy',
#      'name': constants.APP_AUTHOR,
#      'role': 'end-user',
#      'verified': True},
#     ])
# print(json.dumps(obj=status,
#                  indent=4))



##
#     Project: PyZendesk
# Description: API for Zendesk
#      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
#   Copyright: 2021-2025 Fabio Castelli
#     License: GPL-3+
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
##

import json
import os

from pyzendesk import Agents as ZendeskAgents


# Instance zendesk object
zendesk = ZendeskAgents(website=os.environ['ZENDESK_SERVER'])
# Authenticate user
zendesk.authenticate(username=os.environ['ZENDESK_USERNAME'],
                     password=os.environ['ZENDESK_PASSWORD'])

# Get the agents count
count = zendesk.count(include_admins=False)
print('agents count found:', count)
count = zendesk.count(include_admins=True)
print('agents+admins count found:', count)

# Search
agents = zendesk.search(include_admins=False, criteria_list=[])
print('agents details:', len(agents['users']))
agents = zendesk.search(include_admins=True, criteria_list=[])
print('agents+admins details:', len(agents['users']))
# Search all
agents = zendesk.search_all(include_admins=False, criteria_list=[])
print('agents details:', len(agents['users']))
agents = zendesk.search_all(include_admins=True, criteria_list=[])
print('agents+admins details:', len(agents['users']))

# Get agent details
agent = zendesk.get(agent_id=agents['users'][0]['id'])
print(json.dumps(obj=agent,
                 indent=4))

# Get agent related details
agent = zendesk.get_related(agent_id=agents['users'][0]['id'])
print(json.dumps(obj=agent,
                 indent=4))

# Get many agents
agents = zendesk.get_many(agent_ids=[agents['users'][0]['id'],
                                     agents['users'][1]['id'],
                                     agents['users'][2]['id']])
print('agents details:', len(agents['users']))



##
#     Project: PyZendesk
# Description: API for Zendesk
#      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
#   Copyright: 2021-2025 Fabio Castelli
#     License: GPL-3+
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
##

from typing import Any, Optional

from .api import Api

TICKET_STATUS_NEW = 'new'
TICKET_STATUS_OPEN = 'open'
TICKET_STATUS_PENDING = 'pending'
TICKET_STATUS_HOLD = 'hold'
TICKET_STATUS_SOLVED = 'solved'
TICKET_STATUS_CLOSED = 'closed'


class Tickets(Api):
    def get(self, ticket_id: int) -> dict:
        """
        Get a ticket details

        :param ticket_id: ticket ID to get data from
        :return: dictionary with the ticket details
        """
        return self.request_get(path=f'tickets/{ticket_id}.json')

    def get_comments(self, ticket_id: int) -> dict:
        """
        Get a ticket comments

        :param ticket_id: ticket ID to get data from
        :return: dictionary with the ticket details
        """
        return self.request_get(path=f'tickets/{ticket_id}/comments.json')

    def get_comments_all(self, ticket_id: int) -> dict:
        """
        Get all ticket comments

        :param ticket_id: ticket ID to get data from
        :return: dictionary with the ticket details
        """
        results = {}
        current_page = 0
        next_page_url = 'initial value'
        # Copy the criteria list which will be changed during the loop
        while next_page_url:
            current_page += 1
            search_results = self.request_get(
                path=f'tickets/{ticket_id}/comments.json?page={current_page}')
            if not results:
                # First page of results
                results = search_results
            elif 'error' in search_results:
                # Too many results, search interrupted server side
                results['error'] = search_results['error']
                results['description'] = search_results['description']
            else:
                # Append results
                results['comments'].extend(search_results['comments'])
            if 'error' not in search_results:
                # Continue processing the next page
                next_page_url = search_results['next_page']
            else:
                # Stop search if any error occurred
                next_page_url = None
        return results

    def count(self, criteria_list: list) -> Optional[int]:
        """
        Get the number of tickets matching the specified criterias

        :param criteria_list: list of string criterias
        :return: number of tickets found
        """
        criteria = ' '.join(criteria_list)
        results = self.request_get(
            path=f'search/count?query=type:ticket {criteria}')
        return results.get('count')

    def search(self, criteria_list: list) -> dict:
        """
        Get the tickets matching the specified criterias

        :param criteria_list: list of string criterias
        :return: dictionary with tickets details found
        """
        criteria = ' '.join(criteria_list)
        return self.request_get(
            path=f'search?query=type:ticket {criteria}')

    def search_all(self, criteria_list: list) -> dict:
        """
        Get the tickets matching the specified criterias processing all the
        results by requesting also the next pages

        :param criteria_list: list of string criterias
        :return: dictionary with tickets details found
        """
        results = {}
        current_page = 0
        next_page_url = 'initial value'
        # Copy the criteria list which will be changed during the loop
        criteria_list = criteria_list.copy()
        while next_page_url:
            current_page += 1
            criteria_list.append(f'&page={current_page}')
            search_results = self.search(criteria_list=criteria_list)
            if not results:
                # First page of results
                results = search_results
            elif 'error' in search_results:
                # Too many results, search interrupted server side
                results['error'] = search_results['error']
                results['description'] = search_results['description']
            else:
                # Append results
                results['results'].extend(search_results['results'])
            if 'error' not in search_results:
                # Continue processing the next page
                next_page_url = search_results['next_page']
            else:
                # Stop search if any error occurred
                next_page_url = None
            criteria_list.remove(f'&page={current_page}')
        return results

    def search_export(self, criteria_list: list) -> dict:
        """
        Get the tickets matching the specified criterias
        using the search export API

        :param criteria_list: list of string criterias
        :return: dictionary with tickets details found
        """
        criteria = ' '.join(criteria_list)
        return self.request_get(
            path=f'search/export?filter[type]=ticket&query={criteria}')

    def search_export_all(self, criteria_list: list) -> dict:
        """
        Get the tickets matching the specified criterias processing all the
        results by requesting also the next pages using the search export API

        :param criteria_list: list of string criterias
        :return: dictionary with tickets details found
        """
        results = {}
        next_token = 'initial value'
        # Copy the criteria list which will be changed during the loop
        criteria_list = criteria_list.copy()
        while next_token:
            search_results = self.search_export(criteria_list=criteria_list)
            if not results:
                # First page of results
                results = search_results
            elif 'error' in search_results:
                # Too many results, search interrupted server side
                results['error'] = search_results['error']
                results['description'] = search_results['description']
            else:
                # Append results
                results['results'].extend(search_results['results'])
            if f'&page[after]={next_token}' in criteria_list:
                criteria_list.remove(f'&page[after]={next_token}')
            if search_results['meta']['has_more']:
                # Continue processing the next page
                next_token = search_results['meta']['after_cursor']
            else:
                # Stop search if any error occurred
                next_token = None
            criteria_list.append(f'&page[after]={next_token}')
        return results

    def add_comment(self,
                    ticket_id: int,
                    public: bool,
                    text: str,
                    attachments: Optional[list[str]],
                    status: str = None) -> dict:
        """
        Add a public comment to a ticket

        :param ticket_id: ticket ID to update
        :param public: boolean value to make the comment public
        :param text: text to add to the ticket
        :param attachments: list of tokens for attached files
        :param status: new status after the saving the comment
        :return: updated ticket details
        """
        ticket_data = {
            'ticket': {
                'comment': {
                    'public': public,
                    'body': text,
                    'uploads': attachments
                }
            }
        }
        if status is not None:
            ticket_data['ticket']['status'] = status
        return self.request_put(path=f'tickets/{ticket_id}.json',
                                json=ticket_data)

    def add_private_comment(self,
                            ticket_id: int,
                            text: str,
                            attachments: Optional[list[str]],
                            status: str = None) -> dict:
        """
        Add a private comment to a ticket

        :param ticket_id: ticket ID to update
        :param text: text to add to the ticket
        :param attachments: list of tokens for attached files
        :param status: new status after the saving the comment
        :return: updated ticket details
        """
        return self.add_comment(ticket_id=ticket_id,
                                public=False,
                                text=text,
                                attachments=attachments,
                                status=status)

    def add_public_comment(self,
                           ticket_id: int,
                           text: str,
                           attachments: Optional[list[str]],
                           status: str = None) -> dict:
        """
        Add a public comment to a ticket

        :param ticket_id: ticket ID to update
        :param text: text to add to the ticket
        :param attachments: list of tokens for attached files
        :param status: new status after the saving the comment
        :return: updated ticket details
        """
        return self.add_comment(ticket_id=ticket_id,
                                public=True,
                                text=text,
                                attachments=attachments,
                                status=status)

    def set_status(self, ticket_id: int, status: str) -> dict:
        """
        Update ticket status

        :param ticket_id: ticket ID to update
        :param status: new ticket status
        :return: updated ticket details
        """
        return self.request_put(path=f'tickets/{ticket_id}.json',
                                json={
                                    'ticket': {
                                        'status': status
                                    }
                                })

    def get_custom_field(self,
                         ticket: dict,
                         field_id: int,
                         default: Any) -> Optional[Any]:
        """
        Get a custom field value from a Zendesk ticket body

        :param ticket: dictionary with ticket body
        :param field_id: field ID
        :param default: default value for value not found
        :return: custom field value or default value
        """
        return ([field['value']
                 for field in ticket['custom_fields']
                 if field['id'] == field_id] or [default])[0]

    def update_custom_fields(self, ticket_id: int, fields: dict) -> dict:
        """
        Update custom fields for a ticket

        :param ticket_id: ticket ID to update
        :param fields: dictionary object with key as field ID
        :return: updated ticket details
        """
        data = [{'id': key, 'value': value}
                for key, value in fields.items()]
        return self.request_put(path=f'tickets/{ticket_id}.json',
                                json={
                                    'ticket': {
                                        'custom_fields': data
                                    }
                                })

    def get_requester_email(self, ticket: dict) -> Optional[str]:
        """
        Get the sender address from a ticket dictionary.
        In the case the ticket object doesn't contain a valid address try to
        process a new Zendesk search using a previous ticket followup

        :param ticket: dictionary with ticket body
        :return: requester email address
        """
        try:
            from_data = ticket['via']['source']['from']
            if 'address' in from_data:
                # Requester email address
                result = from_data['address'].lower()
            elif 'ticket_id' in from_data:
                # Missing requester address, check in the referenced ticket
                search_results = self.get(ticket_id=from_data['ticket_id'])
                from_data = search_results['ticket']['via']['source']['from']
                result = from_data['address'].lower()
            else:
                # Missing fields for email address
                raise KeyError
        except KeyError:
            result = None
        return result



##
#     Project: PyZendesk
# Description: API for Zendesk
#      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
#   Copyright: 2021-2025 Fabio Castelli
#     License: GPL-3+
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
##

from typing import Optional

from .api import Api


class Users(Api):
    def me(self) -> dict:
        """
        Zendesk requester user information

        :return: user information
        """
        return self.request_get(path='users/me.json')

    def autocomplete(self, name: str) -> dict:
        """
        Get the users with matching name

        :param name: user name to match
        :return: dictionary with the user details
        """
        return self.request_get(path=f'users/autocomplete?name={name}')

    def autocomplete_all(self, name: str) -> dict:
        """
        Get all the users with matching name

        :param name: user name to match
        :return: dictionary with the user details
        """
        results = {}
        current_page = 0
        next_page_url = 'initial value'
        while next_page_url:
            current_page += 1
            search_results = self.autocomplete(name=f'{name}'
                                                    f'&page={current_page}')
            if not results:
                # First page of results
                results = search_results
            elif 'error' in search_results:
                # Too many results, search interrupted server side
                results['error'] = search_results['error']
                results['description'] = search_results['description']
            else:
                # Append results
                results['users'].extend(search_results['users'])
            if 'error' not in search_results:
                # Continue processing the next page
                next_page_url = search_results['next_page']
            else:
                # Stop search if any error occurred
                next_page_url = None
        return results

    def get(self, user_id: int) -> dict:
        """
        Get a user's details

        :param user_id: user ID to get data from
        :return: dictionary with the user details
        """
        return self.request_get(path=f'users/{user_id}')

    def get_many(self, user_ids: list[int]) -> dict:
        """
        Get many users' details

        :param user_ids: list of users ID to get data from
        :return: dictionary with the user details
        """
        ids = ','.join(map(str, user_ids))
        return self.request_get(path=f'users/show_many?ids={ids}')

    def get_related(self, user_id: int) -> dict:
        """
        Get a user's related details

        :param user_id: user ID to get data from
        :return: dictionary with the user related details
        """
        return self.request_get(path=f'users/{user_id}/related')

    def merge(self, user_id: int, user_id_final: int) -> dict:
        """
        Merge a user into another

        :param user_id: user ID to get data from
        :param user_id_final: final user ID to merge data to
        :return: dictionary with the user related details
        """
        return self.request_put(path=f'users/{user_id}/merge',
                                json={'user': {
                                    'id': user_id_final}})

    def get_deleted(self, user_id: int) -> dict:
        """
        Get a deleted user's details

        :param user_id: user ID to get data from
        :return: dictionary with the user related details
        """
        return self.request_get(path=f'deleted_users/{user_id}')

    def list_deleted(self) -> dict:
        """
        Get the deleted users list

        :return: dictionary with the deleted users details
        """
        return self.request_get(path='deleted_users')

    def count(self, criteria_list: list) -> Optional[int]:
        """
        Get the number of users matching the specified criterias

        :param criteria_list: list of string criterias
        :return: number of users found
        """
        criteria = ' '.join(criteria_list)
        results = self.request_get(
            path=f'users/search?query={criteria}')
        return results.get('count')

    def search(self, criteria_list: list) -> dict:
        """
        Get the users matching the specified criterias

        :param criteria_list: list of string criterias
        :return: dictionary with users details found
        """
        criteria = ' '.join(criteria_list)
        return self.request_get(
            path=f'users/search?query={criteria}')

    def search_all(self, criteria_list: list) -> dict:
        """
        Get the users matching the specified criterias processing all the
        results by requesting also the next pages.

        :param criteria_list: list of string criterias
        :return: dictionary with users details found
        """
        results = {}
        current_page = 0
        next_page_url = 'initial value'
        # Copy the criteria list which will be changed during the loop
        criteria_list = criteria_list.copy()
        while next_page_url:
            current_page += 1
            criteria_list.append(f'&page={current_page}')
            search_results = self.search(criteria_list=criteria_list)
            if not results:
                # First page of results
                results = search_results
            elif 'error' in search_results:
                # Too many results, search interrupted server side
                results['error'] = search_results['error']
                results['description'] = search_results['description']
            else:
                # Append results
                results['users'].extend(search_results['users'])
            if 'error' not in search_results:
                # Continue processing the next page
                next_page_url = search_results['next_page']
            else:
                # Stop search if any error occurred
                next_page_url = None
            criteria_list.remove(f'&page={current_page}')
        return results

    def create(self, user: dict) -> dict:
        """
        Create a new user

        :param user: user details dictionary
        :return: updated user details
        """
        return self.request_post(path='users',
                                 json={'user': user})

    def create_many(self, users: list[dict]) -> dict:
        """
        Create many new users

        :param user: user details dictionary
        :return: updated users details
        """
        return self.request_post(path='users/create_many',
                                 json={'users': users})

    def create_or_update(self, user: dict) -> dict:
        """
        Create a new user or update an existing user

        :param user: user details dictionary
        :return: updated user details
        """
        return self.request_post(path='users/create_or_update',
                                 json={'user': user})

    def create_or_update_many(self, users: list[dict]) -> dict:
        """
        Create many new users or update many existing users

        :param users: user details list of dictionaries
        :return: updated user details
        """
        return self.request_post(path='users/create_or_update_many',
                                 json={'users': users})

    def delete(self, user_id: int) -> dict:
        """
        Delete a user

        :param user_id: user ID to delete
        :return: deleted user details
        """
        return self.request_delete(path=f'users/{user_id}')

    def purge(self, user_id: int) -> dict:
        """
        Permanently delete a deleted user

        :param user_id: user ID to delete permanently
        :return: deleted user details
        """
        return self.request_delete(path=f'deleted_users/{user_id}')

    def update(self, user_id: int, user: dict) -> dict:
        """
        Updated an existing user

        :param user_id: user ID to update
        :param user: user details dictionary
        :return: updated user details
        """
        return self.request_put(path=f'users/{user_id}',
                                json={'user': user})
